#include <stm32f0xx.h>
#include <gpio.hpp>
using namespace stm32f072xb;
// https://www.st.com/resource/en/reference_manual/dm00031936-stm32f0x1stm32f0x2stm32f0x8-advanced-armbased-32bit-mcus-stmicroelectronics.pdf
// https://www.st.com/resource/en/datasheet/stm32f072c8.pdf

void setupChannel()
{
	//LED
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN); //enable gpioa (p.120) ... pin is PA5 (datasheet p.37) ok
	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER5_0);
	SET_BIT(GPIOA->MODER, GPIO_MODER_MODER5_1);		//alternate mode (p.157) ok
	GPIOA->AFR[0] = (GPIO_AFRL_AFSEL5 & 0x0200000); //enable alternate function for pin A5 (p.161)
	CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_5);		// Push-pull output (p.157)
}

void setupPWM()
{
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM2EN);				   //enable TIM2 (p.125)
	CLEAR_BIT(TIM2->CCMR1, TIM_CCMR1_OC1PE);				   // output compare 1(p.416 and p.446)
	CLEAR_BIT(TIM2->CR1, TIM_CR1_DIR);						   //upcounting mode (p.435)
	CLEAR_BIT(TIM2->CR1, TIM_CR1_ARPE);						   //enable auto-reload (p.435)
	CLEAR_BIT(TIM2->CCMR1, TIM_CCMR1_CC1S);					   //set chanel 1 as output (p.446)
	SET_BIT(TIM2->CCER, TIM_CCER_CC1E);						   // OC1 is active high (p.450)
	SET_BIT(TIM2->CCMR1, TIM_CCMR1_OC1M_1 | TIM_CCMR1_OC1M_2); //enable pwm mode 1(channel 1) and output compare 1(p.416 and p.446)
	CLEAR_BIT(TIM2->CCER, TIM_CCER_CC1P);					   // enable output signal polarity of channel 1 (p.450)

	SET_BIT(TIM2->EGR, TIM_EGR_UG); //initialise all registers (p.416)

	WRITE_REG(TIM2->PSC, 48);
	WRITE_REG(TIM2->ARR, 1000);
	SET_BIT(TIM2->CR1, TIM_CR1_CEN); //enable counter (p.435)
}

void setPWM(uint32_t speed)
{
	WRITE_REG(TIM2->CCR1, speed);
}

int main()
{
	setupChannel();
	setupPWM();
	uint32_t speed = 250;
	setPWM(speed);
	while (true)
		;
	return 0;
}
