#include <stm32f0xx.h>

/*--------------------------------------------------------------------------------------------------*/
// SOLUTION 1
/*--------------------------------------------------------------------------------------------------*/
const uint32_t SYSTEM_CORE_CLOCK = 8000000;
const uint32_t TICK_FREQ = 1U;
const uint32_t MAX_DELAY = 0xFFFFFFFFU;

volatile uint32_t currentTick = 0;

uint32_t getCurrentTick() {
	return currentTick;
}

void setCurrentTick(uint32_t newTick) {
	currentTick = newTick;
}

extern "C" void SysTick_Handler() {
	uint32_t tick = getCurrentTick();
	if (tick == MAX_DELAY) {
		setCurrentTick(0);
	} else {
		setCurrentTick(tick + TICK_FREQ);
	}
}

void setupSystemClock() {
	WRITE_REG(SysTick->LOAD,
	          (uint32_t)((SYSTEM_CORE_CLOCK / (1000U / TICK_FREQ)) -
	                     1UL));        /* set reload register */
	NVIC_SetPriority(SysTick_IRQn, 0); /* set Priority for Systick Interrupt */
	WRITE_REG(SysTick->VAL, 0UL);      /* Load the SysTick Counter Value */
	WRITE_REG(
	    SysTick->CTRL,
	    SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk |
	        SysTick_CTRL_ENABLE_Msk); /* Enable SysTick IRQ and SysTick Timer */
}

void blockingDelayMs(const uint32_t delay) {
	uint32_t tickStart = getCurrentTick();
	uint32_t wait;

	if (delay < TICK_FREQ) {
		wait = (uint32_t)(TICK_FREQ);
	} else {
		wait =
		    ((uint32_t)(delay) / (uint32_t)(TICK_FREQ)) * (uint32_t)(TICK_FREQ);
	}

	uint32_t goal;

	if (tickStart > (MAX_DELAY - wait)) {
		goal = wait - (MAX_DELAY - tickStart);
	} else {
		goal = tickStart + wait;
	}

	if (wait < MAX_DELAY) {
		wait += (uint32_t)(TICK_FREQ);
	}

	while (getCurrentTick() != goal)
		;
}

/*--------------------------------------------------------------------------------------------------*/
// COMMON SETUP
/*--------------------------------------------------------------------------------------------------*/

void setupLed() {
	// LED
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN);  // enable GPIOA (p.120)
	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER5);
	SET_BIT(GPIOA->MODER, GPIO_MODER_MODER5_0);  // Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_5);  // Push-pull output (p.157)
}

/*--------------------------------------------------------------------------------------------------*/
// MAIN
/*--------------------------------------------------------------------------------------------------*/

int main() {
	// Solution 1
	setupLed();
	setupSystemClock();

	constexpr uint32_t delayLength = 500;

	while (true) {
		// Solution 1
		SET_BIT(GPIOA->ODR, GPIO_ODR_5);  // turn on LED, p.159
		blockingDelayMs(delayLength);
		CLEAR_BIT(GPIOA->ODR, GPIO_ODR_5);  // turn off LED, p.159
		blockingDelayMs(delayLength);
	}
}
